package hmi.dom.base;

import com.google.gwt.core.shared.GwtIncompatible;
import java.io.IOException;
import java.io.InputStream;
import javax.xml.namespace.NamespaceContext;
import javax.xml.namespace.QName;
import javax.xml.xpath.XPathExpressionException;
import org.w3c.dom.Document;

/**
 * Synchronized version of the XmlJavaDocHelper
 * @author AXW09011
 */
@GwtIncompatible
public class SynchronizedXmlJavaDocHelper implements XmlDocHelper {

    private final XmlJavaDocHelper helper;
    private final Object mutex;

    public SynchronizedXmlJavaDocHelper() {
        this.helper = new XmlJavaDocHelper();
        this.mutex = this;
    }

    @Override
    public Document createDocument() {
        synchronized (mutex) {
            return helper.createDocument();
        }
    }

    @Override
    public Object evaluate(NamespaceContext nsContext, String xpath, Object node, QName qname) throws XPathExpressionException {
        synchronized (mutex) {
            return helper.evaluate(nsContext, xpath, node, qname);
        }
    }

    @Override
    public Document parseDocument(String source) throws ParseException {
        synchronized (mutex) {
            return helper.parseDocument(source);
        }
    }

    @Override
    public Document parseDocument(InputStream source, String schemaName) throws ParseException, ValidateException, IOException {
        synchronized (mutex) {
            return helper.parseDocument(source, schemaName);
        }
    }

    @Override
    public String transformToString(Document doc) {
        synchronized (mutex) {
            return helper.transformToString(doc);
        }
    }

    @Override
    public void validate(String data, String schemaName) throws ValidateException {
        synchronized (mutex) {
            helper.validate(data, schemaName);
        }
    }
}
