package hmi.dom.base;

import java.io.IOException;
import java.io.InputStream;
import javax.xml.namespace.NamespaceContext;
import javax.xml.namespace.QName;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpressionException;
import org.w3c.dom.Document;

/**
 * Interface to create and write a DOM document to use with GWT and Java.
 *
 * @author AXW09011
 */
public interface XmlDocHelper {

    /**
     * Create an empty org.w3c.dom.Document
     *
     * @return created document.
     */
    Document createDocument();

    /**
     * Parse the source to an org.w3c.dom.Document
     *
     * @param source XML data.
     * @return document with the data from the source.
     * @throws ParseException If the source cannot be successfully parsed to a document.
     */
    Document parseDocument(String source) throws ParseException;

    /**
     * Validate and parse the source to a document.
     *
     * @param data XML data to validate.
     * @param schemaName Schema name. <code>Null</code> is allowed to avoid schema validation. Loaded with {@link Class#getResource(java.lang.String)
     * }
     * @throws ValidateException Validation failed.
     * @throws ParseException Parsing failed.
     * @throws IOException Run out of data.
     */
    Document parseDocument(InputStream source, String schemaName) throws IOException, ParseException, ValidateException;

    /**
     * Serialize the document to a string.
     * @param doc Document to serialize to a string.
     * @return the document in a string.
     */
    String transformToString(Document doc);

    /**
     * Validation the XML data against the schema.
     * @param data XML data to validate.
     * @param schemaName Schema name, loaded with {@link Class#getResource(java.lang.String)}
     * @throws ValidateException Validation failed.
     */
    void validate(String data, String schemaName) throws ValidateException;
    
    /**
     * Evaluate the compiled XPath expression in the context of the specified InputSource and return the result is a NodeSet.
     * @param nsContext
     * @param xpath
     * @param node
     * @return 
     * @throws javax.xml.xpath.XPathExpressionException
     */
    Object evaluate(NamespaceContext nsContext, String xpath, Object node, QName qname) throws XPathExpressionException;
}