/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package hmi.dom.base;

import com.google.gwt.core.shared.GwtIncompatible;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;
import javax.xml.XMLConstants;
import javax.xml.namespace.NamespaceContext;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Helper methods for a {@link org.w3c.dom.Document }
 *
 * @author AXW09011
 */
@GwtIncompatible
public class XmlJavaDocHelper implements XmlDocHelper {

    private final DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();

    /**
     * Cache for Schemas.
     */
    private static final Map<String, Schema> CACHE = new HashMap<>();

    private DocumentBuilder p;

    {
        try {
            docFactory.setNamespaceAware(true);
            p = docFactory.newDocumentBuilder();
        } catch (ParserConfigurationException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public Document createDocument() {
        return p.newDocument();
    }

    @Override
    public Document parseDocument(String source) throws ParseException {
        try {
            return p.parse(new InputSource(new StringReader(source)));
        } catch (SAXException ex) {
            throw new ParseException(ex);
        } catch (IOException ex) {
            // throw as RuntimeException, because this this cannot be happen
            // since the source is a String.
            throw new RuntimeException(ex);
        }
    }

    @Override
    public Document parseDocument(InputStream source, String schemaName) throws ParseException, ValidateException, IOException {
        try {
            ByteArrayOutputStream data = null;
            if (schemaName != null) {
                data = new ByteArrayOutputStream();
                byte[] buffer = new byte[8096];
                while (true) {
                    int read = source.read(buffer);
                    if (read == -1) {
                        break;
                    }
                    data.write(buffer, 0, read);
                }
                try {
                    validateXML(new StreamSource(new ByteArrayInputStream(data.toByteArray())), loadSchema(schemaName));
                } catch (SAXException | IOException e) {
                    throw new ValidateException(e, "", schemaName);
                }
            }
            return p.parse(data == null ? source : new ByteArrayInputStream(data.toByteArray()));
        } catch (SAXException ex) {
            throw new ParseException(ex);
        }
    }

    @Override
    public void validate(String data, String schemaName) throws ValidateException {
        try {
            validateXML(new StreamSource(new StringReader(data)), loadSchema(schemaName));
        } catch (SAXException | IOException e) {
            throw new ValidateException(e, data, schemaName);
        }
    }

    private static void validateXML(Source source, Schema schema) throws SAXException, IOException {
        schema.newValidator().setProperty(XMLConstants.ACCESS_EXTERNAL_DTD, "");
        schema.newValidator().validate(source);
    }

    private static Schema loadSchema(String schemaName) throws SAXException {
        Schema cached = CACHE.get(schemaName);
        if (cached == null) {
            cached = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI).newSchema(XmlJavaDocHelper.class.getResource(schemaName));
            CACHE.put(schemaName, cached);
        }
        return cached;
    }

    @Override
    public String transformToString(Document doc) {
        try {
            TransformerFactory tFactory = TransformerFactory.newInstance();
            Transformer transformer = tFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
            DOMSource source = new DOMSource(doc);
            StringWriter writer = new StringWriter();
            StreamResult result = new StreamResult(writer);
            transformer.transform(source, result);
            return writer.toString();
        } catch (TransformerConfigurationException ex) {
            throw new RuntimeException(ex);
        } catch (TransformerException ex) {
            throw new RuntimeException(ex);
        }
    }

    @Override
    public Object evaluate(NamespaceContext nsContext, String xpath, Object node, QName qname) throws XPathExpressionException {
        XPathFactory fact = XPathFactory.newInstance();            
        XPath xPath = fact.newXPath();    
        xPath.setNamespaceContext(nsContext);
        return xPath.compile(xpath).evaluate(node, qname);
    }    
    
}
